@@ -2,6 +2,7 @@
 import sys
 import os.path
 import itertools
+import platform
 
 from doit import dependency
 from doit.task import Task
@@ -14,16 +15,25 @@ from doit.dependency import Dependency
 def doit_run(dependencyFile, task_list, output, options=None,
              verbosity=None, alwaysExecute=False, continue_=False,
              reporter='default', num_process=0):
+    """
+    @param reporter: (str) one of provided reporters or ...
+                     (class) user defined reporter class (can only be specified
+           from DOIT_CONFIG - never from command line)
+    """
     # get tasks to be executed
     task_control = TaskControl(task_list)
     task_control.process(options)
 
     # reporter
-    if reporter not in REPORTERS:
-        msg = ("No reporter named '%s'.\nType 'doit help run' to see a list "
-               "of available reporters.")
-        raise InvalidCommand(msg % reporter)
-    reporter_cls = REPORTERS[reporter]
+    if isinstance(reporter, str):
+        if reporter not in REPORTERS:
+            msg = ("No reporter named '%s'.\nType 'doit help run' to see a list "
+                   "of available reporters.")
+            raise InvalidCommand(msg % reporter)
+        reporter_cls = REPORTERS[reporter]
+    else:
+        # user defined class
+        reporter_cls = reporter
 
     # verbosity
     if verbosity is None:
@@ -223,15 +233,27 @@ class FileModifyWatcher(object):
     2) create an object passing a list of files to be watched
     3) call the loop method
     """
+    supported_platforms = ('Darwin', 'Linux')
+
     def __init__(self, file_list):
         """@param file_list (list-str): files to be watched"""
         self.file_list = set([os.path.abspath(f) for f in file_list])
         self.watch_dirs = set([os.path.dirname(f) for f in self.file_list])
         self.notifier = None
+        self.platform = platform.system()
+        if self.platform not in self.supported_platforms:
+            msg = "Unsupported platform '%s'\n" % self.platform
+            msg += ("'auto' command is supported only on %s" %
+                    (self.supported_platforms,))
+            raise Exception(msg)
 
     def _handle(self, event):
-        if event.pathname in self.file_list:
-            self.handle_event(event)
+        if self.platform == 'Darwin':
+            if event.name in self.file_list:
+                self.handle_event(event)
+        elif self.platform == 'Linux':
+            if event.pathname in self.file_list:
+                self.handle_event(event)
 
     def handle_event(self, event):
         """this should be sub-classed """
@@ -241,21 +263,48 @@ class FileModifyWatcher(object):
         """Infinite loop
         @loop_callback: used to stop loop on unittests
         """
-        import pyinotify
-        handler = self._handle
-        class EventHandler(pyinotify.ProcessEvent):
-            def process_default(self, event):
-                handler(event)
 
-        wm = pyinotify.WatchManager()  # Watch Manager
-        mask = pyinotify.IN_CLOSE_WRITE
-        ev = EventHandler()
-        self.notifier = pyinotify.Notifier(wm, ev)
+        if self.platform == 'Darwin':
+            from fsevents import Observer
+            from fsevents import Stream
+            from fsevents import IN_MODIFY
 
-        for watch_this in self.watch_dirs:
-            wm.add_watch(watch_this, mask)
+            observer = Observer()
+            handler = self._handle
+            def fsevent_callback(event):
+                if event.mask == IN_MODIFY:
+                    handler(event)
 
-        self.notifier.loop(loop_callback)
+            for watch_this in self.watch_dirs:
+                stream = Stream(fsevent_callback, watch_this, file_events=True)
+                observer.schedule(stream)
+
+            observer.daemon = True
+            observer.start()
+            try:
+                # hack to keep main thread running...
+                import time
+                while True:
+                    time.sleep(99999)
+            except (SystemExit, KeyboardInterrupt):
+                pass
+
+        elif self.platform == 'Linux':
+            import pyinotify
+            handler = self._handle
+            class EventHandler(pyinotify.ProcessEvent):
+                def process_default(self, event):
+                    handler(event)
+
+            wm = pyinotify.WatchManager()  # Watch Manager
+            mask = pyinotify.IN_CLOSE_WRITE
+            ev = EventHandler()
+            self.notifier = pyinotify.Notifier(wm, ev)
+
+            for watch_this in self.watch_dirs:
+                wm.add_watch(watch_this, mask)
+
+            self.notifier.loop(loop_callback)
 
 
 def doit_auto(dependency_file, task_list, filter_tasks, loop_callback=None):
@@ -266,7 +315,7 @@ def doit_auto(dependency_file, task_list, filter_tasks, loop_callback=None):
     """
     task_control = TaskControl(task_list)
     task_control.process(filter_tasks)
-    tasks_to_run = list(set([t for t in task_control.get_next_task(True)]))
+    tasks_to_run = list(set([t for t in task_control.task_dispatcher(True)]))
     watch_tasks = [t.name for t in tasks_to_run]
     watch_files = list(itertools.chain(*[s.file_dep for s in tasks_to_run]))
     watch_files = list(set(watch_files))
