@@ -8,39 +8,6 @@ from doit.exceptions import TaskFailed, SetupError, DependencyError
 from doit.dependency import Dependency
 from doit.task import Task
 
-class SetupManager(object):
-    """Manage setup objects
-
-    Setup object is any object that implements 'setup' and/or 'cleanup'
-    @ivar _loaded (list): of loaded setup objects
-    """
-
-    def __init__(self):
-        self._loaded = set()
-
-
-    def load(self, setup_obj):
-        """run setup from a setup_obj if it is not loaded yet"""
-        if setup_obj in self._loaded:
-            return
-
-        self._loaded.add(setup_obj)
-        if hasattr(setup_obj, 'setup'):
-            setup_obj.setup()
-
-
-    def cleanup(self):
-        """run cleanup for all loaded objects"""
-        for setup_obj in self._loaded:
-            if hasattr(setup_obj, 'cleanup'):
-                try:
-                    setup_obj.cleanup()
-                # report error but keep result as successful.
-                # FIXME should execute all cleanup's even with errors
-                # TODO caller should handle the exception
-                except Exception, e:
-                    return SetupError("ERROR on setup_obj cleanup", e)
-
 
 # execution result.
 SUCCESS = 0
@@ -72,7 +39,6 @@ class Runner(object):
         self.always_execute = always_execute
         self.verbosity = verbosity
 
-        self.setupManager = SetupManager()
         self.teardown_list = [] # list of tasks to be teardown
         self.final_result = SUCCESS # until something fails
         self._stop_running = False
@@ -94,12 +60,18 @@ class Runner(object):
     def select_task(self, task):
         """Returns bool, task should be executed
          * side-effect: set task.options
+
+         Tasks should be executed if they are not up-to-date.
+         Tasks that cointains setup-tasks must be selected twice,
+         so it gives chance for other tasks to be executed after
+         checking it is not up-to-date.
         """
 
-        # check if run_status was already calculated
+        # if run_status is not None, it was already calculated
         if task.run_status is None:
             # TODO reporter.start_task rename to get_status
             self.reporter.start_task(task)
+
             # check if task is up-to-date
             try:
                 task.run_status = self.dependencyManager.get_status(task)
@@ -112,6 +84,7 @@ class Runner(object):
                 # if task is up-to-date skip it
                 if task.run_status == 'up-to-date':
                     self.reporter.skip_uptodate(task)
+                    task.values = self.dependencyManager.get_values(task.name)
                     return False
                 # check if task should be ignored (user controlled)
                 if task.run_status == 'ignore':
@@ -122,10 +95,9 @@ class Runner(object):
                 # dont execute now, execute setup first...
                 return False
         else:
+            # sanity checks
             assert task.run_status == 'run', "%s:%s" % (task.name, task.run_status)
-            # check if already executed
-            if not task.setup_tasks:
-                return False
+            assert task.setup_tasks
 
         # selected just need to get values from other tasks
         for arg, value in task.getargs.iteritems():
@@ -142,15 +114,7 @@ class Runner(object):
 
     def execute_task(self, task):
         """execute task's actions"""
-        # setup env
-        for setup_obj in task.setup:
-            try:
-                self.setupManager.load(setup_obj)
-            except (SystemExit, KeyboardInterrupt): raise
-            except Exception, exception:
-                return SetupError("ERROR on object setup", exception)
-
-        # new style cleanup/teardown
+        # register cleanup/teardown
         if task.teardown:
             self.teardown_list.append(task)
 
@@ -160,6 +124,7 @@ class Runner(object):
 
 
     def process_task_result(self, task, catched_excp):
+        task.run_status = "done"
         # save execution successful
         if catched_excp is None:
             self.dependencyManager.save_success(task)
@@ -177,7 +142,7 @@ class Runner(object):
 
         @param task_control: L{TaskControl}
         """
-        for task in task_control.get_next_task():
+        for task in task_control.task_dispatcher():
             if self._stop_running:
                 break
             if not self.select_task(task):
@@ -203,11 +168,6 @@ class Runner(object):
         # flush update dependencies
         self.dependencyManager.close()
 
-        # clean setup objects
-        error = self.setupManager.cleanup()
-        if error:
-            self.reporter.cleanup_error(error)
-
         # new style teardown
         self.teardown()
 
@@ -274,7 +234,7 @@ class MP_Runner(Runner):
                 return None
 
         while True:
-            # get new task
+            # get new task from ready queue
             if self.ready_queue:
                 task_name = self.ready_queue.pop(0)
                 task = self.tasks[task_name]
@@ -287,9 +247,19 @@ class MP_Runner(Runner):
                 except StopIteration:
                     return nothing_ready()
 
+
+            # task with setup must be selected twice...
+            wait_for = task.task_dep + task.calc_dep
+            # must wait for setup_tasks too if on second select.
+            if not (task.setup_tasks and task.run_status is None):
+                wait_for += task.setup_tasks
+
             # check task-dependencies are done
-            for dep in task.task_dep + task.setup_tasks:
-                if self.tasks[dep].run_status == 'run':
+            for dep in  wait_for:
+                if (self.tasks[dep].run_status is None or
+                    self.tasks[dep].run_status == 'run'):
+                    # not ready yet, add to waiting dict and re-start loop
+                    # to get another task
                     if dep in self.waiting:
                         self.waiting[dep].append(task.name)
                     else:
@@ -301,35 +271,40 @@ class MP_Runner(Runner):
                     return task
 
     def set_tasks(self, task_control):
-        self.task_gen = task_control.get_next_task()
+        self.task_gen = task_control.task_dispatcher()
         self.tasks = task_control.tasks
 
-    def _finished_running_task(self, task):
-        task.run_status = "done"
+    def process_task_result(self, task, catched_excp):
+        Runner.process_task_result(self, task, catched_excp)
         if task.name in self.waiting:
             for ready_task in self.waiting[task.name]:
                 self.ready_queue.append(ready_task)
             del self.waiting[task.name]
 
-    def run_tasks(self, task_control):
-        result_q = Queue()
-        task_q = Queue()
+    def _run_start_processes(self, task_q, result_q):
         proc_list = []
-        self.set_tasks(task_control)
-
-        # create and start processes
         for p_id in xrange(self.num_process):
             next_task = self.get_next_task()
             if next_task is None:
                 break # do not start more processes than tasks
             if isinstance(next_task, Task):
-                task_q.put(next_task.name)
-            else:
-                task_q.put(next_task)
+                task_q.put((next_task.name, next_task.file_dep))
+            else: # next_task is a string
+                # no task ready to be executed but some are on the queue
+                # awaiting to be executed
+                task_q.put((next_task, None))
             process = Process(target=self.execute_task,
                               args=(task_q, result_q))
             process.start()
             proc_list.append(process)
+        return proc_list
+
+    def run_tasks(self, task_control):
+        result_q = Queue()
+        task_q = Queue()
+        self.set_tasks(task_control)
+        # create and start processes
+        proc_list = self._run_start_processes(task_q, result_q)
 
         # wait for all processes terminate
         proc_count = len(proc_list)
@@ -350,7 +325,6 @@ class MP_Runner(Runner):
 
             # completed one task, dispatch next one
             self.process_task_result(task, catched_excp)
-            self._finished_running_task(task)
 
             free_proc = self.free_proc
             self.free_proc = 0
@@ -359,9 +333,23 @@ class MP_Runner(Runner):
                 if next_task is None:
                     proc_count -= 1
                 if isinstance(next_task, Task):
-                    task_q.put(next_task.name)
+                    task_q.put((next_task.name, next_task.file_dep))
                 else:
-                    task_q.put(next_task)
+                    task_q.put((next_task, None))
+
+
+            # check for cyclic dependencies
+            if len(proc_list) == self.free_proc:
+                msg = "Cyclic dependencies on tasks:\n"
+                for task_name, wait_list in self.waiting.iteritems():
+                    msg += (" * Task %s is waiting for tasks %s\n" %
+                            (task_name, ", ".join(wait_list)))
+                self.final_result = ERROR
+                self.reporter.runtime_error(msg)
+                # terminate all child process
+                proc_count = 0
+                for proc in proc_list:
+                    task_q.put((None, None))
 
         # we are done, join all process
         for proc in proc_list:
@@ -382,7 +370,7 @@ class MP_Runner(Runner):
 
         try:
             while True:
-                task_name = task_q.get()
+                task_name, file_dep = task_q.get()
                 if task_name is None:
                     self.teardown()
                     return # no more tasks to execute finish this process
@@ -391,13 +379,9 @@ class MP_Runner(Runner):
                     continue
 
                 task = self.tasks[task_name]
+                task.file_dep = file_dep
                 result = {'name': task.name}
                 # FIXME support setup objects with 2 "scopes" (global and process)
-                if task.setup:
-                    raise Exception("Task '%s' has Setup-objects. " % task.name +
-                                    "Setup-objects are deprecated and not" +
-                                    " supported with Multi-processing.")
-
                 t_result = Runner.execute_task(self, task)
 
                 if t_result is None:
@@ -405,10 +389,9 @@ class MP_Runner(Runner):
                     result['values'] = task.values
                 else:
                     result['failure'] = t_result
-
                 result_q.put(result)
         except (SystemExit, KeyboardInterrupt, Exception), e:
             # error, blow-up everything
-            result_q.put({'name': task.name,
+            result_q.put({'name': task_name,
                           'exit': e.__class__,
                           'exception': str(e)})
