@@ -17,12 +17,12 @@ class Task(object):
     @ivar task_dep: (list - string)
     @ivar wild_dep: (list - string) task dependency using wildcard *
     @ivar file_dep: (list - string)
+    @ivar calc_dep: (list - string) reference to a task
+    @ivar calc_dep_stack: (list - string) unprocessed calc_dep
     @ivar dep_changed (list - string): list of file-dependencies that changed
           (are not up_to_date). this must be set before
     @ivar run_once: (bool) task without dependencies should run
     @ivar run_always: (bool) task always run even if up-to-date
-    @ivar setup (list): List of setup objects
-          (any object with setup or cleanup method)
     @ivar setup_tasks (list - string): references to task-names
     @ivar is_subtask: (bool) indicate this task is a subtask.
     @ivar result: (str) last action "result". used to check task-result-dep
@@ -43,7 +43,10 @@ class Task(object):
     # list of valid types/values for each task attribute.
     valid_attr = {'name': [str],
                   'actions': [list, tuple, None],
-                  'dependencies': [list, tuple],
+                  'file_dep': [list, tuple],
+                  'task_dep': [list, tuple],
+                  'result_dep': [list, tuple],
+                  'calc_dep': [list, tuple],
                   'targets': [list, tuple],
                   'setup': [list, tuple],
                   'clean': [list, tuple, True],
@@ -57,7 +60,8 @@ class Task(object):
 
     setup_warned = False # print setup-object deprecation warning just once
 
-    def __init__(self, name, actions, dependencies=(), targets=(),
+    def __init__(self, name, actions, file_dep=(), targets=(),
+                 task_dep=(), result_dep=(), calc_dep=(),
                  setup=(), clean=(), teardown=(), is_subtask=False, doc=None,
                  params=(), verbosity=None, title=None, getargs=None):
         """sanity checks and initialization
@@ -80,18 +84,7 @@ class Task(object):
         self.verbosity = verbosity
         self.custom_title = title
         self.getargs = getargs
-
-        self.setup = [] # old setup objects
-        self.setup_tasks = [] # new setup tasks
-        for setup_item in setup:
-            if isinstance(setup_item, str):
-                self.setup_tasks.append(setup_item)
-            else:
-                self.setup.append(setup_item)
-                if not self.__class__.setup_warned:
-                    print "DEPRECATION WARNING: Setup-objects are deprecated. ",
-                    print "Use setup-tasks and task.teardown instead."
-                    self.__class__.setup_warned = True
+        self.setup_tasks = list(setup)
 
         # options
         self.taskcmd = cmdparse.TaskOption(name, params, None, None)
@@ -120,50 +113,76 @@ class Task(object):
         for action in self.actions:
             action.task = self
 
-        self._init_dependencies(dependencies)
+        # dependencies
+        self.dep_changed = None
+
+        self.file_dep = []
+        self._expand_file_dep(file_dep)
+
+        # task_dep
+        self.task_dep = []
+        self.wild_dep = []
+        self._expand_task_dep(task_dep)
+
+        # result_dep
+        self.result_dep = []
+        self._expand_result_dep(result_dep)
+
+        # calc_dep
+        self.calc_dep = []
+        self.calc_dep_stack = []
+        self._expand_calc_dep(calc_dep)
+
         self._init_getargs()
         self._init_doc(doc)
 
         self.run_status = None
 
-
-    def _init_dependencies(self, dependencies):
-        self.dep_changed = None
-        # there are 3 kinds of dependencies: file, task, result
-        self.task_dep = []
-        self.wild_dep = []
-        self.file_dep = []
-        self.result_dep = []
-        for dep in dependencies:
+    def _expand_file_dep(self, file_dep):
+        for dep in file_dep:
             # bool
             if isinstance(dep,bool):
                 if dep is True:
                     self.run_once = True
                 if dep is False:
                     self.run_always = True
-            # string
-            elif isinstance(dep,str):
-                # task dep starts with a ':'
-                if dep.startswith(':'):
-                    if "*" in dep:
-                        self.wild_dep.append(dep[1:])
-                    else:
-                        self.task_dep.append(dep[1:])
-                # task-result dep starts with a '?'
-                elif dep.startswith('?'):
-                    # result_dep are also task_dep.
-                    self.task_dep.append(dep[1:])
-                    self.result_dep.append(dep[1:])
-                # file dep
-                else:
-                    self.file_dep.append(dep)
             # ignore None values
             elif dep is None:
                 continue
             else:
-                msg = ("%s. Invalid paramater in 'dependencies' "+
-                       "got:'%s(%s)'")
-                raise InvalidTask(msg%(self.name, str(dep), type(dep)))
+                # TODO store file_deps in a set
+                if dep not in self.file_dep:
+                    self.file_dep.append(dep)
+
+    def _expand_task_dep(self, task_dep):
+        for dep in task_dep:
+            if "*" in dep:
+                self.wild_dep.append(dep)
+            else:
+                self.task_dep.append(dep)
+
+    def _expand_result_dep(self, result_dep):
+        self.result_dep.extend(result_dep)
+        # result_dep are also task_dep
+        self.task_dep.extend(self.result_dep)
+
+    def _expand_calc_dep(self, calc_dep):
+        for dep in calc_dep:
+            if dep not in self.calc_dep:
+                self.calc_dep.extend(calc_dep)
+                self.calc_dep_stack.extend(calc_dep)
+
+
+    def update_deps(self, deps):
+        for dep, dep_values in deps.iteritems():
+            if dep == 'task_dep':
+                self._expand_task_dep(dep_values)
+            elif dep == 'file_dep':
+                self._expand_file_dep(dep_values)
+            elif dep == 'result_dep':
+                self._expand_result_dep(dep_values)
+            elif dep == 'calc_dep':
+                self._expand_calc_dep(dep_values)
 
 
     def _init_getargs(self):
@@ -314,6 +333,7 @@ class Task(object):
         return "<Task: %s>"% self.name
 
 
+DEPRECATION = False # already printed deprecation?
 def dict_to_task(task_dict):
     """Create a task instance from dictionary.
 
@@ -323,6 +343,7 @@ def dict_to_task(task_dict):
     @param task_dict (dict): task representation as a dict.
     @raise InvalidTask: If unexpected fields were passed in task_dict
     """
+    global DEPRECATION
 
     # check required fields
     if 'actions' not in task_dict:
@@ -330,8 +351,35 @@ def dict_to_task(task_dict):
                           (task_dict['name'],task_dict))
 
     # user friendly. dont go ahead with invalid input.
-    for key in task_dict.keys():
-        if key not in Task.valid_attr.keys():
+    task_attrs = task_dict.keys()
+    for key in task_attrs:
+
+        if key == 'dependencies':
+            if not DEPRECATION:
+                print ("DEPRECATION WARNING: task attribute 'dependencies': "
+                       "use 'file_dep', 'task_dep', 'result_dep' or 'calc_dep'.")
+                DEPRECATION = True
+            task_dict['file_dep'] = []
+            task_dict['task_dep'] = []
+            task_dict['result_dep'] = []
+            task_dict['calc_dep'] = []
+            for dep in task_dict['dependencies']:
+                if isinstance(dep, str) or isinstance(dep, unicode):
+                    # task dep starts with a ':'
+                    if dep.startswith(':'):
+                        task_dict['task_dep'].append(dep[1:])
+                    # task-result dep starts with a '?'
+                    elif dep.startswith('?'):
+                        task_dict['result_dep'].append(dep[1:])
+                    # file dep
+                    else:
+                        task_dict['file_dep'].append(dep)
+                else:
+                    # bool or None
+                    task_dict['file_dep'].append(dep)
+            del task_dict['dependencies']
+
+        elif key not in Task.valid_attr.keys():
             raise InvalidTask("Task %s contains invalid field: '%s'"%
                               (task_dict['name'],key))
 
