@@ -186,10 +186,10 @@ class TestSaveSuccess(object):
         t1 = Task('t1', None)
         t1.result = "result"
         depfile.save_success(t1)
-        t2 = Task('t2', None, ['?t1'])
+        t2 = Task('t2', None, result_dep=['t1'])
         depfile.save_success(t2)
         assert get_md5(t1.result) == depfile._get("t2", "task:t1")
-        t3 = Task('t3', None, [':t1'])
+        t3 = Task('t3', None, task_dep=['t1'])
         depfile.save_success(t3)
         assert None is depfile._get("t3", "task:t1")
 
@@ -197,25 +197,39 @@ class TestSaveSuccess(object):
         t1 = Task('t1', None)
         t1.values = {'x':5, 'y':10}
         depfile.save_success(t1)
-        assert 5 == depfile._get("t1", ":x:")
-        assert 10 == depfile._get("t1", ":y:")
+        assert {'x':5, 'y':10} == depfile._get("t1", "_values_:")
 
 
 class TestGetValue(object):
+    def test_all_values(self, depfile):
+        t1 = Task('t1', None)
+        t1.values = {'x':5, 'y':10}
+        depfile.save_success(t1)
+        assert {'x':5, 'y':10} == depfile.get_values('t1')
+
     def test_ok(self, depfile):
-        depfile._set('t1', ':x:', 5)
+        t1 = Task('t1', None)
+        t1.values = {'x':5, 'y':10}
+        depfile.save_success(t1)
         assert 5 == depfile.get_value('t1.x')
 
     def test_invalid_string(self, depfile):
+        t1 = Task('t1', None)
+        t1.values = {'x':5, 'y':10}
+        depfile.save_success(t1)
         py.test.raises(Exception, depfile.get_value, 'nono')
 
     def test_invalid_taskid(self, depfile):
-        depfile._set('t1', ':x:', 5)
+        t1 = Task('t1', None)
+        t1.values = {'x':5, 'y':10}
+        depfile.save_success(t1)
         py.test.raises(Exception, depfile.get_value, 'nonono.x')
 
     def test_invalid_arg(self, depfile):
-        depfile._set('t1', ':x:', 5)
-        py.test.raises(Exception, depfile.get_value, 't1.y')
+        t1 = Task('t1', None)
+        t1.values = {'x':5, 'y':10}
+        depfile.save_success(t1)
+        py.test.raises(Exception, depfile.get_value, 't1.z')
 
 
 class TestRemoveSuccess(object):
@@ -387,7 +401,7 @@ class TestGetStatus(object):
         # t1 ok, but t2 not saved
         t1 = Task('t1', None)
         t1.result = "result"
-        t2 = Task('t2', None, ['?t1'])
+        t2 = Task('t2', None, result_dep=['t1'])
         depfile.save_success(t1)
         assert 'run' == depfile.get_status(t2)
 
