@@ -54,7 +54,7 @@ class TestRunner_SelectTask(object):
 
     def test_DependencyError(self,reporter):
         t1 = Task("taskX", [(my_print, ["out a"] )],
-                  dependencies=["i_dont_exist"])
+                  file_dep=["i_dont_exist"])
         my_runner = runner.Runner(TESTDB, reporter)
         assert False == my_runner.select_task(t1)
         assert ('start', t1) == reporter.log.pop(0)
@@ -62,7 +62,7 @@ class TestRunner_SelectTask(object):
         assert not reporter.log
 
     def test_upToDate(self,reporter):
-        t1 = Task("taskX", [(my_print, ["out a"] )], dependencies=[__file__])
+        t1 = Task("taskX", [(my_print, ["out a"] )], file_dep=[__file__])
         my_runner = runner.Runner(TESTDB, reporter)
         my_runner.dependencyManager.save_success(t1)
         assert False == my_runner.select_task(t1)
@@ -93,9 +93,6 @@ class TestRunner_SelectTask(object):
         assert True == my_runner.select_task(t1)
         assert ('start', t1) == reporter.log.pop(0)
         assert not reporter.log
-        # trying to select again fails and prints nothing
-        assert False == my_runner.select_task(t1)
-        assert not reporter.log
 
     def test_withSetup(self, reporter):
         t1 = Task("taskX", [(my_print, ["out a"] )], setup=["taskY"])
@@ -289,7 +286,7 @@ class TestRunner_All(object):
 
     def test_resultDependency(self, reporter, RunnerClass):
         t1 = Task("t1", [(ok,)])
-        t2 = Task("t2", [(ok,)], ['?t1'])
+        t2 = Task("t2", [(ok,)], result_dep=['t1'])
         my_runner = RunnerClass(TESTDB, reporter)
         tc = TaskControl([t1, t2])
         tc.process(None)
@@ -304,7 +301,7 @@ class TestRunner_All(object):
 
         # again
         t1 = Task("t1", [(ok,)])
-        t2 = Task("t2", [(ok,)], ['?t1'])
+        t2 = Task("t2", [(ok,)], result_dep=['t1'])
         my_runner2 = RunnerClass(TESTDB, reporter)
         tc2 = TaskControl([t1, t2])
         tc2.process(None)
@@ -318,7 +315,7 @@ class TestRunner_All(object):
 
         # change t1, t2 executed again
         t1B = Task("t1", [(ok2,)])
-        t2 = Task("t2", [(ok,)], ['?t1'])
+        t2 = Task("t2", [(ok,)], result_dep=['t1'])
         my_runner3 = RunnerClass(TESTDB, reporter)
         tc3 = TaskControl([t1B, t2])
         tc3.process(None)
@@ -362,88 +359,6 @@ class TestRunner_All(object):
         py.test.raises(SystemExit, my_runner.run_tasks, tc)
 
 
-class TestTaskSetupObject(object):
-
-    class SetupSample(object):
-        def __init__(self):
-            self.executed = 0
-            self.cleaned = 0
-
-        def setup(self):
-            self.executed += 1
-
-        def cleanup(self):
-            self.cleaned += 1
-
-    def testExecuted(self, reporter):
-        setup = self.SetupSample()
-        t = Task("ss", None, [], [], [setup])
-        my_runner = runner.Runner(TESTDB, reporter)
-        tc = TaskControl([t])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        assert runner.SUCCESS == my_runner.finish()
-        assert 1 == setup.executed
-        assert 1 == setup.cleaned
-
-    def testExecuteOnce(self, reporter):
-        setup = self.SetupSample()
-        t1 = Task("ss", None, [], [], [setup])
-        t2 = Task("ss2", None, [], [], [setup])
-        my_runner = runner.Runner(TESTDB, reporter)
-        tc = TaskControl([t1, t2])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        assert runner.SUCCESS == my_runner.finish()
-        assert 1 == setup.executed
-        assert 1 == setup.cleaned
-
-    def testExecuteCleanedOnTaskErrors(self, reporter):
-        setup = self.SetupSample()
-        def bad_seed():
-            raise Exception("rrrr")
-        t1 = Task("ss", [bad_seed], [], [], [setup])
-        t2 = Task("ss2", None, [], [], [setup])
-        my_runner = runner.Runner(TESTDB, reporter)
-        tc = TaskControl([t1, t2])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        assert runner.ERROR == my_runner.finish()
-        assert 1 == setup.executed
-        assert 1 == setup.cleaned
-
-    def testSetupError(self, reporter):
-        # it is same as a task error
-        def raise_something():
-            raise Exception('xxx')
-        setup = self.SetupSample()
-        setup.setup = raise_something
-        t1 = Task('t1', None, [], [], [setup])
-        my_runner = runner.Runner(TESTDB, reporter)
-        tc = TaskControl([t1])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        assert runner.ERROR == my_runner.finish()
-        # TODO not checking error is written to output
-
-    def testCleanupError(self, reporter):
-        # ignore errors...
-        def raise_something():
-            raise Exception('xxx')
-        setup = self.SetupSample()
-        setup.cleanup = raise_something
-        t1 = Task('t1', None, [], [], [setup])
-        my_runner = runner.Runner(TESTDB, reporter)
-        tc = TaskControl([t1])
-        tc.process(None)
-        my_runner.run_tasks(tc)
-        assert runner.SUCCESS == my_runner.finish()
-        assert ('start', t1) == reporter.log.pop(0)
-        assert ('execute', t1) == reporter.log.pop(0)
-        assert ('success', t1) == reporter.log.pop(0)
-        assert ('cleanup_error',) == reporter.log.pop(0)
-
-
 class TestMP_Reporter(object):
     class MyRunner(object):
         def __init__(self):
@@ -490,7 +405,7 @@ class TestMP_Runner_get_next_task(object):
 
     def test_waiting(self, reporter):
         t1 = Task('t1', [])
-        t2a = Task('t2A', [], dependencies=(':t1',))
+        t2a = Task('t2A', [], task_dep=('t1',))
         t2b = Task('t2B', [], setup=('t1',))
         t3 = Task('t3', [], setup=('t2B',))
         tc = TaskControl([t1, t2a, t2b, t3])
@@ -504,14 +419,14 @@ class TestMP_Runner_get_next_task(object):
         # hold until t1 finishes
         assert 0 == run.free_proc
         assert isinstance(run.get_next_task(), runner.Hold)
-        assert {'t1':['t2A', 't2B']} == run.waiting
+        assert {'t1':['t2A', 't2B'], 't2B': ['t3']} == run.waiting
         assert 1 == run.free_proc
 
         # ready for t2x
         assert [] == run.ready_queue
-        run._finished_running_task(t1)
+        run.process_task_result(t1, None)
         assert ['t2A', 't2B'] == run.ready_queue
-        assert {} == run.waiting
+        assert {'t2B': ['t3']} == run.waiting
 
         # t2
         assert t2a == run.get_next_task()
@@ -519,6 +434,10 @@ class TestMP_Runner_get_next_task(object):
 
         # t3
         assert isinstance(run.get_next_task(), runner.Hold)
-        run._finished_running_task(t2b)
+        run.process_task_result(t2b, None)
+        assert ['t3'] == run.ready_queue
+        assert {} == run.waiting
         assert t3 == run.get_next_task()
         assert None == run.get_next_task()
+
+# test cyclic dependency on MP
